@@ -1,7 +1,14 @@
 # _*_ coding: utf-8 _*_
+from .interfaces import IFhirResourceExtractor
 from collective.elasticsearch.brain import BrainFactory
+from fhirpath.engine.base import EngineResult
+from fhirpath.engine.base import EngineResultBody
+from fhirpath.engine.base import EngineResultHeader
+from fhirpath.engine.base import EngineResultRow
 from fhirpath.search import Search
+from plone.app.fhirfield import IFhirResourceValue
 from Products.ZCatalog.Lazy import LazyMap
+from zope.component import queryMultiAdapter
 
 import math
 
@@ -78,23 +85,42 @@ class ElasticResult(object):
             return self.results[result_key][result_index]
 
 
-def zcatalog_fhir_search(context, query_string=None, params=None):
+def build_engine_result(lazy_maps):
+    """ """
+    total = 0
+    body = EngineResultBody()
+
+    for brain in lazy_maps:
+        row = EngineResultRow()
+        extractor = queryMultiAdapter((brain.getObject(),), IFhirResourceExtractor)
+        assert extractor
+        val = extractor()
+        if IFhirResourceValue.providedBy(val):
+            val = val.foreground_origin()
+        row.append(val)
+        body.append(row)
+        total += 1
+
+    result = EngineResult(header=EngineResultHeader(total=total), body=body)
+    return result
+
+
+def zcatalog_fhir_search(
+    context, query_string=None, params=None, bundle_response=False
+):
     """ """
     query_result = Search(
         context=context, query_string=query_string, params=params
     ).build()
 
     query_copy = query_result._query.clone()
-    resource_type = context.resource_name
-    field_index_name = context.engine.calculate_field_index_name(resource_type)
-
     if context.unrestricted is False:
         context.engine.build_security_query(query_copy)
 
     params = {
         "query": query_copy,
-        "root_replacer": field_index_name,
-        "mapping": context.engine.get_mapping(resource_type),
+        "calculate_field_index_name": context.engine.calculate_field_index_name,
+        "get_mapping": context.engine.get_mapping,
     }
 
     compiled = context.engine.dialect.compile(**params)
@@ -110,4 +136,11 @@ def zcatalog_fhir_search(context, query_string=None, params=None):
     query_params = {"stored_fields": "path.path"}
     result = ElasticResult(context.engine.es_catalog, compiled, **query_params)
     factory = BrainFactory(context.engine.es_catalog.catalog)
-    return LazyMap(factory, result, result.count)
+    lazy_maps = LazyMap(factory, result, result.count)
+
+    if bundle_response is False:
+        return lazy_maps
+
+    engine_result = build_engine_result(lazy_maps)
+
+    return context.engine.wrapped_with_bundle(engine_result)
